/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Original Code is DigitalSesame
 * Portions created by DigitalSesame are Copyright (C) 1997-2000 DigitalSesame
 * All Rights Reserved.
 * 
 * Contributor(s):
 *     Rex Tsai <chihchun@digitalsesame.com>
 *     David Li <david@digitalsesam.com>
 * 
 * $Id: XHTMLTableSectionElementImpl.java,v 1.1.1.1 2003/03/10 16:36:11 taweili Exp $
 */

package org.enhydra.xml.xhtml.dom.xerces;

import org.w3c.dom.*;
import org.w3c.dom.html.*;

import org.enhydra.xml.xhtml.dom.*;

public class XHTMLTableSectionElementImpl 
    extends XHTMLElementImpl 
    implements org.enhydra.xml.xhtml.dom.XHTMLTableSectionElement 
{

    public XHTMLTableSectionElementImpl (XHTMLDocumentBase owner, String namespaceURI, String tagName) {
	super( owner, namespaceURI, tagName);
    }

        public void setId (String newValue) {
	setAttribute("id", newValue);
    }

    public String getId () {
	return getAttribute ("id");
    }
    public void setLang (String newValue) {
	setAttribute("lang", newValue);
    }

    public String getLang () {
	return getAttribute ("lang");
    }
    public void setDir (String newValue) {
	setAttribute("dir", newValue);
    }

    public String getDir () {
	return getAttribute ("dir");
    }
    public void setClassName (String newValue) {
	setAttribute("class", newValue);
    }

    public String getClassName () {
	return getAttribute ("class");
    }
    public void setTitle (String newValue) {
	setAttribute("title", newValue);
    }

    public String getTitle () {
	return getAttribute ("title");
    }
    public void setAlign (String newValue) {
	setAttribute("align", newValue);
    }

    public String getAlign () {
	return getAttribute ("align");
    }

    public void setVAlign (String newValue) {
	setAttribute("valign", newValue);
    }

    public String getVAlign () {
	return getAttribute ("valign");
    }
    public void setOnKeyUp (String newValue) {
	setAttribute("onkeyup", newValue);
    }

    public String getOnKeyUp () {
	return getAttribute ("onkeyup");
    }
    public void setStyle (String newValue) {
	setAttribute("style", newValue);
    }

    public String getStyle () {
	return getAttribute ("style");
    }
    public void setOnMouseDown (String newValue) {
	setAttribute("onmousedown", newValue);
    }

    public String getOnMouseDown () {
	return getAttribute ("onmousedown");
    }
    public void setOnKeyPress (String newValue) {
	setAttribute("onkeypress", newValue);
    }

    public String getOnKeyPress () {
	return getAttribute ("onkeypress");
    }
    public void setOnDblClick (String newValue) {
	setAttribute("ondblclick", newValue);
    }

    public String getOnDblClick () {
	return getAttribute ("ondblclick");
    }
    public void setOnKeyDown (String newValue) {
	setAttribute("onkeydown", newValue);
    }

    public String getOnKeyDown () {
	return getAttribute ("onkeydown");
    }
    public void setOnMouseMove (String newValue) {
	setAttribute("onmousemove", newValue);
    }

    public String getOnMouseMove () {
	return getAttribute ("onmousemove");
    }
    public void setOnMouseUp (String newValue) {
	setAttribute("onmouseup", newValue);
    }

    public String getOnMouseUp () {
	return getAttribute ("onmouseup");
    }
    public void setXmlLang (String newValue) {
	setAttribute("xml:lang", newValue);
    }

    public String getXmlLang () {
	return getAttribute ("xml:lang");
    }
    public void setOnMouseOut (String newValue) {
	setAttribute("onmouseout", newValue);
    }

    public String getOnMouseOut () {
	return getAttribute ("onmouseout");
    }
    public void setOnClick (String newValue) {
	setAttribute("onclick", newValue);
    }

    public String getOnClick () {
	return getAttribute ("onclick");
    }
    public void setOnMouseOver (String newValue) {
	setAttribute("onmouseover", newValue);
    }

    public String getOnMouseOver () {
	return getAttribute ("onmouseover");
    }
    public void setChOff (String newValue) {
	setAttribute("charoff", newValue);
    }

    public String getChOff () {
	return getAttribute ("charoff");
    }

    public String getCh() {
        String    ch;
        
        // Make sure that the access key is a single character.
        ch = getAttribute( "char" );
        if ( ch != null && ch.length() > 1 )
            ch = ch.substring( 0, 1 );
        return ch;
    }
    
    
    public void setCh( String ch ) {
        // Make sure that the access key is a single character.
        if ( ch != null && ch.length() > 1 )
            ch = ch.substring( 0, 1 );
        setAttribute( "char", ch );
    }
    
    public HTMLCollection getRows() {
        if ( _rows == null )
            _rows = new XHTMLCollectionImpl( this, XHTMLCollectionImpl.ROW );
        return _rows;
    }
    
    
    public HTMLElement insertRow( int index ) {
        XHTMLTableRowElementImpl    newRow;
        
        newRow = new XHTMLTableRowElementImpl( (XHTMLDocumentBase) getOwnerDocument(), getNamespaceURI(), "TR" );
        newRow.insertCell( 0 );
        if ( insertRowX( index, newRow ) >= 0 )
            appendChild( newRow );
        return newRow;
    }
    
    
    int insertRowX( int index, XHTMLTableRowElementImpl newRow ) {
        Node    child;
        
        child = getFirstChild();
        while ( child != null ) {
            if ( child instanceof HTMLTableRowElement ) {
                if ( index == 0 ) {
                    insertBefore( newRow, child );
                    return -1;
                }
                --index;
            }
            child = child.getNextSibling();
        }
        return index;
    }

    
    public void deleteRow( int index ) {
        deleteRowX( index );
    }

    
    int deleteRowX( int index ) {
        Node    child;
        
        child = getFirstChild();
        while ( child != null ) {
            if ( child instanceof HTMLTableRowElement ) {
                if ( index == 0 ) {
                    removeChild ( child );
                    return -1;
                }
                --index;
            }
            child = child.getNextSibling();
        }
        return index;
    }

    private XHTMLCollectionImpl    _rows;



;
}


