/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: XMLCOptionsParser.java,v 1.1.1.1 2003/03/10 16:36:20 taweili Exp $
 */

package org.enhydra.xml.xmlc.commands.xmlc;

import org.enhydra.xml.xmlc.XMLCException;
import org.enhydra.xml.xmlc.XMLCError;
import org.enhydra.xml.xmlc.metadata.*;
import org.enhydra.xml.xmlc.commands.options.*;
import org.enhydra.xml.io.ErrorReporter;

import java.io.Reader;
import java.io.File; 
import java.io.IOException;
import java.util.Hashtable;
import java.util.Vector;
import java.util.Enumeration;

/**
 * Parser for xmlc command line options. Includes OptionSet that defines
 * the options.  Parses them into a MetaData object.
 */
public class XMLCOptionsParser {
    /**
     * Usage message.
     */
    private static final String USAGE = "wrong # args: xmlc [options] [optfile.xmlc ...] src.ext";

    /**
     * Set of options.
     */
    private OptionSet optionSet = new OptionSet();

    /**
     * Constructor.  Initializes all option objects.
     */
    public XMLCOptionsParser() {
        new CompileOptsCmdOptions(optionSet);
        new CompilerCmdOptions(optionSet);
        new DOMEditCmdOptions(optionSet);
        new DocClassCmdOptions(optionSet);
        new HTMLCmdOptions(optionSet);
        new ParserCmdOptions(optionSet);
    }

    /**
     * Parse command line, returning the metadata object.
     * @param errorReporter Use to report metadata parse errors.
     */
    public MetaData parse(String[] args,
                          ErrorReporter errorReporter)
        throws XMLCException, IOException {
        OptionsParser optParser = new OptionsParser(optionSet,
                                                    errorReporter);
        optParser.parse(args);

        MetaData metaData = optParser.getMetaData();
        CompileOptions compileOptions = metaData.getCompileOptions();
        String[] posArgs = optParser.getPositionalArgs();

        // No filename is required if print version is specified, otherwise
        // must have an input (although it is allowed with -version).
        if (posArgs.length > 1) {
            throw new XMLCException(USAGE);
        } else if (posArgs.length == 0) {
            if (!compileOptions.getPrintVersion()) {
                throw new XMLCException(USAGE);
            }
        } else {
            metaData.getInputDocument().setUrl(posArgs[0]);
            metaData.getDocument().completeModifications();
        }
        return metaData;
    }

}
