/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is David
 * Li.
 *
 * The Enhydra Application Server and portions created by Lutris
 * Technologies, Inc. are Copyright Lutris Technologies, Inc.  All
 * Rights Reserved.
 * 
 * Contributor(s): 
 * David Li
 *
 * $Id: DynamicClassLoader.java,v 1.2 2003/08/02 12:39:02 taweili Exp $
 */
package org.enhydra.xml.xmlc.deferredparsing;

import java.util.Collections;
import java.util.Map;
import java.util.HashMap;
import java.io.File;

import org.enhydra.xml.xmlc.XMLCLogger;
import org.enhydra.xml.xmlc.StreamXMLCLogger;

import org.apache.bcel.generic.ClassGen;
import org.apache.bcel.generic.ConstantPoolGen;

import org.apache.bcel.classfile.JavaClass;
import org.apache.bcel.classfile.ConstantPool;
import org.apache.bcel.classfile.Method;
import org.apache.bcel.Constants;
import org.apache.bcel.Repository;

public class DynamicClassLoader extends ClassLoader
{
    private ClassLoader delegateLoader = null;
    private XMLCLogger logger = null;
    private Map classMap;

    DynamicClassLoader(ClassLoader parent) {
	this(parent, null);
    }
    
    DynamicClassLoader(ClassLoader parent, XMLCLogger logger) {
	super(parent);
	if (logger != null) {
	    this.logger = logger;
	} else {
	    this.logger = new StreamXMLCLogger();
	}
	classMap = Collections.synchronizedMap(new HashMap());
    }

    /**
     * Add a delegate be used in class loading.
     * @param delegate the loader to be used as delegate
     */
    public void setDelegate (ClassLoader delegate) {
	delegateLoader = delegate;
    }

    /**
     * load class, 
     */
    public Class loadClass(String className) throws ClassNotFoundException {
	Class clazz = null;
	ClassLoader loader = null;

	try {
	    if (delegateLoader != null) {
		loader = delegateLoader;
	    } else {
		loader = getParent();
	    }
	    clazz = loader.loadClass(className);
	} catch (ClassNotFoundException e) {
	    if (loader != getParent()) {
		clazz = getParent().loadClass(className);
	    } else {
		throw e;
	    }
	}

	return clazz;
    }

    /**
     * Define a class in this loader using the given JavaClass
     * @param clazz JavaClass to be defined this
     */
    private Class defineClass (JavaClass clazz) {
	byte[] buf = clazz.getBytes();
	return defineClass(null, buf, 0, buf.length);
    }

    /**
     * Generate a class based on the class name
     * @param className class name for the class to be generated
     */
    Class createClass(String className, String path) {

	if (logger.debugEnabled()) {
	    logger.logDebug(">>>Request to create class " + className + " from document " + path);
	}

	Class clazz = (Class)classMap.get(className);
	if (clazz == null) {
	    synchronized(classMap) {
		clazz = (Class)classMap.get(className);
		if (clazz == null) {
		    DynamicMLCreator dhc = new DynamicMLCreator(className, path);
		    JavaClass javaClass = dhc.create();
		    clazz = defineClass(javaClass);
		
		    classMap.put(className, clazz);
		}
	    }
	} else {
	    if (logger.debugEnabled()) {
		logger.logDebug(">>>Got class " + className + "from cache");
	    }
	}

	return clazz;
    }

}
