/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: XercesHTMLDomFactory.java,v 1.1.1.1 2003/03/10 16:36:22 taweili Exp $
 */

package org.enhydra.xml.xmlc.dom.xerces;

import org.enhydra.xml.xmlc.XMLObject;
import org.enhydra.xml.xmlc.XMLObjectLink;
import org.enhydra.xml.xmlc.dom.XMLCDomFactory;
import org.enhydra.xml.xmlc.dom.HTMLDomFactoryMethods;
import org.w3c.dom.Document;
import org.w3c.dom.DocumentType;
import org.w3c.dom.Element;
import org.w3c.dom.html.HTMLDocument;
import org.enhydra.apache.html.dom.HTMLDocumentImpl;

/**
 * XMLC DOM factory for creating HTML Document objects based on the Xerces
 * DOM.
 */
public class XercesHTMLDomFactory extends XercesDomFactory
                                  implements XMLCDomFactory {
    // FIXME: deprecated in 4.0 beta 2, can be removed when object code
    // compatibility XMLC 2.0.1 is no longer required.  This was removed
    // since generate a variable name to an inner class broke forte.
    /**
     * Class that adds XMLObjectLink support to the Xerces HTMLDocument.
     * @deprecated Remains for compaitiblity with objects compiled
     *  with XMLC 2.0.1
     */
    public class LinkedXercesDocument extends HTMLDocumentImpl
                                      implements XMLObjectLink {
        /** Reference to XMLObject containing this Document. */
        private XMLObject fXmlObjectLink;

        /** Constructor */
        public LinkedXercesDocument() {
        }

        /** @see XMLObjectLink#setXMLObject */
        public void setXMLObject(XMLObject xmlObject) {
            fXmlObjectLink = xmlObject;
        }

        /** @see XMLObjectLink#getXMLObject */
        public XMLObject getXMLObject() {
            return fXmlObjectLink;
        }
    }

    /**
     * This creates a Document object that extends the Xerces HTMLDocument
     * class and implements XMLObjectLink.
     * @see XMLCDomFactory#createDocument
     */
    public Document createDocument(String namespaceURI,
                                   String qualifiedName,
                                   DocumentType docType) {
        HTMLDocument doc = new XercesLinkedHTMLDocument();

        // This creates the document element.
        doc.getDocumentElement();
        return doc;
    }

    /**
     * @see XMLCDomFactory#getMIMEType
     */
    public String getMIMEType() {
        return "text/html";
    }

    /**
     * @see XMLCDomFactory#getBaseClassName
     */
    public String getBaseClassName() {
        return HTMLDomFactoryMethods.getBaseClassName();
    }

    /**
     * @see XMLCDomFactory#getInterfaceNames
     */
    public String[] getInterfaceNames() {
        return HTMLDomFactoryMethods.getInterfaceNames();
    }

    /**
     * @see XMLCDomFactory#getElementClassNames
     */
    public String[] getElementClassNames(Element element) {
        return HTMLDomFactoryMethods.getElementClassNames(element);
    }

    /**
     * @see XMLCDomFactory#isURLAttribute
     */
    public boolean isURLAttribute(Element element,
                                  String attrName) {
        return HTMLDomFactoryMethods.isURLAttribute(element, attrName);
    }
}
