/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: SwingHTMLParser.java,v 1.1.1.1 2003/03/10 16:36:22 taweili Exp $
 */

package org.enhydra.xml.xmlc.html.parsers.swing;

import org.enhydra.xml.xmlc.*;
import org.enhydra.xml.xmlc.html.*;
import org.enhydra.xml.xmlc.html.parsers.HTMLParserBase;
import org.enhydra.xml.xmlc.html.parsers.HTMLDocBuilder;
import org.enhydra.xml.xmlc.compiler.*;
import org.enhydra.xml.xmlc.misc.LineNumberMap;
import org.enhydra.xml.xmlc.misc.LineNumberMapReader;
import org.enhydra.xml.xmlc.parsers.ParseTracer;
import org.enhydra.xml.xmlc.parsers.XMLCParser;
import org.enhydra.xml.xmlc.dom.XMLCDocument;
import org.enhydra.xml.xmlc.dom.XMLCDomFactory;
import org.enhydra.xml.xmlc.metadata.MetaData;
import org.enhydra.xml.xmlc.metadata.ParserType;
import org.enhydra.xml.io.ErrorReporter;
import org.enhydra.xml.xmlc.metadata.HTMLSection;
import org.w3c.dom.*;
import org.w3c.dom.html.*;
import org.xml.sax.InputSource;
import java.io.*;
import java.util.StringTokenizer;


// FIXME: should we check that no propritary tags are supplied???


/**
 * XMLCParser object for HTML and HTML framesets using the Swing HTML parser.
 */
public class SwingHTMLParser extends HTMLParserBase implements XMLCParser {
    /**
     * @see XMLCParser#parse
     */
    public XMLCDocument parse(InputSource input,
                              LineNumberMap lineNumberMap,
                              XMLCDomFactory domFactory,
                              MetaData metaData,
                              ErrorReporter errorReporter,
                              ParseTracer tracer)
        throws IOException, XMLCException {
        HTMLDocBuilder docBuilder = new HTMLDocBuilder(domFactory,
                                                       input);
        validateConf(ParserType.SWING, metaData);

        // We need a LineNumberMap, even with a single file, since swing
        // reports things in character offsets.
        Reader reader = null;
        if (lineNumberMap == null) {
            LineNumberMapReader mapReader = new LineNumberMapReader(input);
            reader = mapReader;
            lineNumberMap = mapReader.getLineNumberMap();
        } else {
            reader = input.getCharacterStream();
        }

        SwingParser parser = new SwingParser();
        parser.parse(docBuilder, errorReporter, tracer, reader, lineNumberMap);
        if (errorReporter.getErrorCnt() != 0) {
            handleParseErrors(errorReporter);
        }
        
        addPCDataContentElements(docBuilder.getXMLCDocument());
        return docBuilder.getXMLCDocument();
    }
}
