/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: TestProperties.java,v 1.1.1.1 2003/03/10 16:36:25 taweili Exp $
 */

package org.enhydra.xml.driver;
import java.util.Properties;
import java.io.File;

// FIXME: should validate string properties on get and set

/**
 * Definition and access to properies used by the tests.
 */
public class TestProperties {
    /** Property names (keep doc updated in package.html) */
    public static final String XMLC_TEST_ROOT = "xmlc.test.root";
    public static final String XMLC_TEST_UPDATE = "xmlc.test.update";
    public static final String XMLC_TEST_VERBOSE = "xmlc.test.verbose";
    public static final String XMLC_TEST_PARSER = "xmlc.test.parser";
    public static final String XMLC_TEST_DOM = "xmlc.test.dom";
    public static final String XMLC_TEST_RELOADING = "xmlc.test.reloading";
    public static final String XMLC_TEST_DEFERREDPARSING = "xmlc.test.deferredparsing";
    public static final String XMLC_TEST_JAVAC_CLASSPATH
        = "xmlc.test.javac.classpath";

    /** Get a boolean system property */
    private static boolean getBooleanProp(String name) {
        Properties props = System.getProperties();
        String val = props.getProperty(name, Boolean.FALSE.toString());
        if (val == null) {
            return false;
        } else if (val.equalsIgnoreCase("true")
                   || val.equalsIgnoreCase("on")
                   || val.equalsIgnoreCase("1")) {
            return true;
        } else {
            return false;
        }
    }

    /** Set a boolean system property */
    private static void setBooleanProp(String name,
                                       boolean val) {
        Properties props = System.getProperties();
        props.setProperty(name, (val ? Boolean.TRUE.toString()
                                 : Boolean.FALSE.toString()));
    }

    /** 
     * Set the test case root directory.  If not defined, the current
     * directory is returned.
     */
    public static void setTestRoot(String dir) {
        Properties props = System.getProperties();
        props.setProperty(XMLC_TEST_ROOT, dir);
    }

    /** 
     * Get the test case root directory.  If not defined, the current
     * directory is returned.
     */
    public static File getTestRoot() {
        Properties props = System.getProperties();
        String dir = props.getProperty(XMLC_TEST_ROOT);
        if (dir == null) {
            dir = ".";
        }
        return new File(dir);
    }
    
    /** Get the update property. */
    public static boolean getUpdate() {
        return getBooleanProp(XMLC_TEST_UPDATE);
    }

    /** Set the update property. */
    public static void setUpdate(boolean val) {
        setBooleanProp(XMLC_TEST_UPDATE, val);
    }

    /** Get the verbose property. */
    public static boolean getVerbose() {
        return getBooleanProp(XMLC_TEST_VERBOSE);
    }

    /** Set the verbose property. */
    public static void setVerbose(boolean val) {
        setBooleanProp(XMLC_TEST_VERBOSE, val);
    }

    /** Get the parser property */
    public static String getParser(String defVal) {
        Properties props = System.getProperties();
        return props.getProperty(XMLC_TEST_PARSER, defVal);
    }

    /** Set the parser property */
    public static void setParser(String val) {
        Properties props = System.getProperties();
        props.setProperty(XMLC_TEST_PARSER, val);
    }

    /** Get the DOM property */
    public static String getDom(String defVal) {
        Properties props = System.getProperties();
        return props.getProperty(XMLC_TEST_DOM, defVal);
    }
    
    /** Set the DOM property */
    public static void setDom(String val) {
        Properties props = System.getProperties();
        props.setProperty(XMLC_TEST_DOM, val);
    }
    
    /** Get the reloading property. */
    public static boolean getReloading() {
        return getBooleanProp(XMLC_TEST_RELOADING);
    }

    /** Set the reloading property. */
    public static void setReloading(boolean val) {
        setBooleanProp(XMLC_TEST_RELOADING, val);
    }

    /** Get the deferred parsing property. */
    public static boolean getDeferredParsing() {
        return getBooleanProp(XMLC_TEST_DEFERREDPARSING);
    }

    /** Set the deferred parsing property. */
    public static void setDeferredParsing(boolean val) {
        setBooleanProp(XMLC_TEST_DEFERREDPARSING, val);
    }

    /** Get the javac classpath property */
    public static String getJavacClassPath() {
        Properties props = System.getProperties();
        return props.getProperty(XMLC_TEST_JAVAC_CLASSPATH);
    }
    
    /** Set the javac classpath property */
    public static void setJavacClassPath(String val) {
        Properties props = System.getProperties();
        props.setProperty(XMLC_TEST_JAVAC_CLASSPATH, val);
    }
}
