/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: ParserBase.java,v 1.1.1.1 2003/03/10 16:36:27 taweili Exp $
 */

package org.enhydra.xml.oasis;

import java.util.Stack;
import java.io.IOException;
import org.xml.sax.SAXException;
import org.xml.sax.InputSource;
import org.xml.sax.Attributes;
import org.xml.sax.Locator;
import org.xml.sax.helpers.DefaultHandler;
import org.enhydra.apache.xerces.parsers.SAXParser;
import org.enhydra.xml.io.ErrorReporter;

/**
 * Base class for the SAX parsers used by this package.
 */
class ParserBase extends DefaultHandler {
    /** ErrorHandler to use */
    protected ErrorReporter fErrorReporter;

    /** Current location */
    protected Locator fLocator;

    /** SAX parser */
    private SAXParser parser = new SAXParser();

    /**
     * Constructor.
     */
    protected ParserBase(ErrorReporter errorReporter) throws SAXException {
        fErrorReporter = errorReporter;
        parser.setFeature("http://xml.org/sax/features/validation", true);
        parser.setErrorHandler(fErrorReporter);
        parser.setContentHandler(this);
    }

    /**
     * Receive a Locator object for document events.
     * @see org.xml.sax.ContentHandler#setDocumentLocator
     */
    public void setDocumentLocator(Locator locator) {
        fLocator = locator;
    }

    /**
     * Parse a document.
     */
    protected void doParse(InputSource doc) throws SAXException, IOException {
        parser.parse(doc);
    }

    /**
     * Utility to get an SAX attribute value.
     * @return An interned attribute value or null if the attribute is not
     * found.
     */
    protected static String getAttr(Attributes attrs,
                             String name) {
        String value = attrs.getValue("", name);
        if (value != null) {
            value = value.intern();
        }
        return value;
    }
}
