/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: TestSuiteParser.java,v 1.1.1.1 2003/03/10 16:36:27 taweili Exp $
 */

package org.enhydra.xml.oasis;

import java.util.Stack;
import java.io.IOException;
import java.io.File;
import org.xml.sax.SAXException;
import org.xml.sax.InputSource;
import org.xml.sax.Attributes;
import org.xml.sax.Locator;
import org.xml.sax.helpers.DefaultHandler;
import org.enhydra.apache.xerces.parsers.SAXParser;
import org.enhydra.xml.io.ErrorReporter;

/**
 * Parser that parses a testcases.dtd file into a TestSuite object.
 */
final class TestSuiteParser extends ParserBase {
    /** Root of the parsed tree */
    private TestSuite fTestSuite;

    /** Parse stack of node objects */
    private Stack fNodeStack = new Stack();

    /** Test directory */
    private File fTestDir;
    
    /**
     * Constructor.
     */
    TestSuiteParser(File testDir,
                    ErrorReporter errorReporter) throws SAXException {
        super(errorReporter);
        fTestDir = testDir;
    }

    /**
     * Append to the description of the object on the 
     * top of the stack.
     */
    private void appendDescription(String desc) {
        Object tos = fNodeStack.peek();
        if (tos instanceof TestCases) {
            ((TestCases)tos).appendDescription(desc);
        } else {
            ((Test)tos).appendDescription(desc);
        }
    }

    /**
     * Start a new element.
     * @see org.xml.sax.ContentHandler#startElement
     */
    public void startElement(String uri,
                             String localName,
                             String rawName,
                             Attributes attributes) throws SAXException {
        if (localName.equals("TESTSUITE")) {
            fTestSuite = new TestSuite(fTestDir, attributes);
            fNodeStack.push(fTestSuite);
        } else if (localName.equals("TESTCASES")) {
            Object tos = fNodeStack.peek();
            TestCases testCases = ((TestCases)tos).createTestCases(attributes);
            fNodeStack.push(testCases);
            fTestSuite.recordTestCases(testCases);
        } else if (localName.equals("TEST")) {
            Test test = ((TestCases)fNodeStack.peek()).createTest(attributes);
            fNodeStack.push(test);
        } else if (localName.equals("EM")) {
            appendDescription("<EM>");
        } else if (localName.equals("B")) {
            appendDescription("<B>");
        } else {
            fErrorReporter.error("Unknown element \"" + localName + "\"",
                                 fLocator);
        }
    }
    
    /**
     * Handle end of element.
     * @see org.xml.sax.ContentHandler#endElement
     */
    public void endElement(String uri,
                           String localName,
                           String rawName) throws SAXException {
        // Didn't push anything for the HTML-like tags
        if (!(localName.equals("EM") || localName.equals("B"))) {
            fNodeStack.pop();
        }
    }
    
    /**
     * Handle characters.
     * @see org.xml.sax.ContentHandler#characters
     */
    public void characters(char ch[],
                           int start,
                           int length) throws SAXException {
        appendDescription(new String(ch, start, length));
    }

    /**
     * Parse an test description..
     */
    TestSuite parse(InputSource doc) throws SAXException, IOException {
        doParse(doc);
        if (fTestSuite == null) {
            throw new SAXException("Failed to parse anything from: "
                                   + doc.getSystemId());
        }
        return fTestSuite;
    }
}
