/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: CornerCasesTests.java,v 1.1.1.1 2003/03/10 16:36:28 taweili Exp $
 */

package org.enhydra.xml.xmlc.html;
import org.enhydra.xml.driver.TestDiff;
import org.enhydra.xml.xmlc.driver.XmlcTestParams;
import org.enhydra.xml.xmlc.driver.ExecXmlc;
import java.lang.reflect.Method;
import junit.framework.Test;

/**
 * Tests of of various special cases, many that caused bugs. Not all work with
 * the swing parser.
 */
public class CornerCasesTests extends HtmlTestCaseBase {
    /** Factory method to create suite of these tests */
    public static Test suite() {
        return createSuite(CornerCasesTests.class, null);
    }
    
    /** Constructor */
    public CornerCasesTests(Method method) {
        super(method);
    }

    /*
     * Test 1: special characters in attributes, urlmappings, and
     * invalid names in form input.
     */
    public void test1() {
        HtmlBasicTest test = createTest("testPage1.html");
        test.addOptionFile(getInputFile("testPage1.xmlc"));
        test.basicTest();
    }

    /*
     * Test 2: head tag parsing
     */
    public void test2() {
        HtmlBasicTest test = createTest("testPage2.html");
        test.basicTest();
    }

    /*
     * Test 3: two forms
     */
    public void test3() {
        HtmlBasicTest test = createTest("Form2.html");
        test.basicTest();
    }

    /*
     * Test 4: weird name attributes
     */
    public void test4() {
        HtmlBasicTest test = createTest("testPage3.html");
        test.basicTest();
    }

    /*
     * Test 5: form contain P tags
     */
    public void test5() {
        // Swing parse can't handle this
        if (!fParams.getParser().equals(ExecXmlc.SWING_PARSER)) {
            HtmlBasicTest test = createTest("formWithPs.html");
            test.basicTest();
        }
    }


    /*
     * Test 6: form containing BR tags
     */
    public void test6() {
        HtmlBasicTest test = createTest("formWithBRs.html");
        test.basicTest();
    }

    /*
     * Test 8: name and class name constant generation
     */
    public void test8() {
        HtmlBasicTest test = createTest("testPage4.html");
        test.basicTest();
    }

    /*
     * Test 9: Swing parser mangled iframes.
     */
    public void test9() {
        HtmlBasicTest test = createTest("iframe.html");
        test.basicTest();
    }

    /*
     * Test 10:  Deleting elements by class
     */
    public void test10() {
        HtmlBasicTest test = createTest("DeleteByClass.html");
        test.addFirst(ExecXmlc.OPT_DELETE_CLASS, "deleteMe");
        test.basicTest();
    }

    /*
     * Test 11: Tests for various parsing related bugs
     */
    public void test11() {
        HtmlBasicTest test = createTest("testPage5.html");
        test.basicTest();
    }

    /*
     * Test 12: Entity problems (attribute values, etc).
     */
    public void test12() {
        HtmlBasicTest test = createTest("testPage6.html");
        test.basicTest();

        // Check different files in reloading tests
        TestDiff differ = getDiffer();
        differ.fgrepMustFind("value=\"&aelig;, &oslash; and &aring;\"",
                             test.getGenerateFile());
        differ.fgrepMustFind("value=\"&aelig;, &oslash; and &aring;\"",
                                 test.getRegenerateFile());
    }

    /*
     * Test 13: Lazy DOM broke when non-standard attribute names
     * contained `:'.  This will not work with the Swing parser.
     */
    public void test13() {
        // Swing parse can't handle this
        if (!fParams.getParser().equals(ExecXmlc.SWING_PARSER)) {
            HtmlBasicTest test = createTest("namespace-like.html");
            test.basicTest();
        }
    }

    /*
     * Test 14:  more non-standard attribute names.
     */
    public void test14() {
        HtmlBasicTest test = createTest("namespace-like2.html");
        test.addOptionFile(getInputFile("namespace-like2.xmlc"));
        if (fParams.getParser().equals(ExecXmlc.SWING_PARSER)) {
            // Swing parse can't handle this document
            test.setCompileShouldFail();
        }
        test.basicTest();
    }

    /*
     * Test 15: Handling of style elements.  This broke in older versions
     * of the Swing parser, which only worked if the contents were in
     * a comment.  It appears to be working now.
     */
    public void test15() {
        HtmlBasicTest test = createTest("style1.html");
        test.basicTest();
    }

    /*
     * Test 16: Style, contents in comment
     */
    public void test16() {
        HtmlBasicTest test = createTest("style2.html");
        test.basicTest();
    }
    /*
     * Test 17: Problems with spacing being different in generated HTML.
     */
    public void test17() {
        HtmlBasicTest test = createTest("space-bug1.html");
        test.basicTest();
    }

    /*
     * Test 18: Someone reported that newlines in <A> body didn't work.
     */
    public void test18() {
        HtmlBasicTest test = createTest("href-spaces.html");
        test.basicTest();
    }

    /*
     * Test 19: Broken document exported from Word caused an endless
     * loop in Tidy parser.  Swing can't handle it.
     */
    public void test19() {
        HtmlBasicTest test = createTest("caused-hang.html");
        if (fParams.getParser().equals(ExecXmlc.SWING_PARSER)) {
            test.setCompileShouldFail();
        }
        test.basicTest();
    }

    /*
     * Test 20: Javascript quoting
     */
    public void test20() {
        HtmlBasicTest test = createTest("javascript2.html");
        test.addCyberStudioTags();
        test.basicTest();
    }
}
