/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: HtmlTestCaseBase.java,v 1.1.1.1 2003/03/10 16:36:28 taweili Exp $
 */

package org.enhydra.xml.xmlc.html;

import org.enhydra.xml.xmlc.driver.XmlcTestCaseBase;
import org.enhydra.xml.xmlc.driver.XmlcTestParams;
import org.enhydra.xml.xmlc.driver.ExecXmlc;
import java.lang.reflect.Method;
import java.io.File;

/**
 * Base class for all XMLC HTML test cases.  Contains methods to
 * assist in setting up individual tests.
 */
public class HtmlTestCaseBase extends XmlcTestCaseBase {
    /** HTML encoding to use */
    private String fHtmlEncoding;

    /** Construct specifying */
    protected HtmlTestCaseBase(Method method) {
        super(method, XmlcTestParams.getParams(false));
    }

    /** Create a new test object */
    protected HtmlBasicTest createTest(String inputDoc) {
        return new HtmlBasicTest(this, getInputFile(inputDoc));
    }

    /** Create a new test object */
    protected HtmlBasicTest createTest(File inputDoc) {
        return new HtmlBasicTest(this, inputDoc);
    }

    /** Create a new test object with the lutris-index.html file */
    protected HtmlBasicTest createLutrisIndexTest() {
        HtmlBasicTest test = createTest("lutris-index.html");
        test.addCyberStudioTags();
        return test;
    }


    /** Set the HTML encoding to use. */
    public void setHtmlEncoding(String encoding) {
        fHtmlEncoding = encoding;
    }

    /**
     * Build a ExecXmlc object with argument required for the initial compile
     * and the recompile/dump
     */
    public void addStdOpts(ExecXmlc execXmlc) {
        if (fHtmlEncoding != null) {
            execXmlc.addOpt(ExecXmlc.OPT_HTML_ENCODING, fHtmlEncoding);
        }
    }
}
