/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: OasisTests.java,v 1.1.1.1 2003/03/10 16:37:11 taweili Exp $
 */

package org.enhydra.xml.xmlc.oasis;

import org.enhydra.xml.xmlc.*;
import org.enhydra.xml.xmlc.compiler.*;
import org.enhydra.xml.xmlc.compiler.Compiler;
import org.enhydra.xml.xmlc.metadata.*;
import org.enhydra.xml.oasis.OasisTestDriver;
import org.enhydra.xml.oasis.Test;
import org.enhydra.xml.io.ErrorReporter;
import java.io.*;
import org.w3c.dom.*;
import org.xml.sax.SAXException;
import org.xml.sax.InputSource;

/**
 * FIXME: this is not working or completeld.  The idea is to compiling
 * all of the Oasis XML test files with XMLC, probably saving and
 * revalidting.  Maybe just using XmlBasicTest to run this.
 *
 * Test XMLC using the OASIS XML parser conformance tests.
 * Command line:
 * <pre>
 *   OasisTests classdir testpkg domfactory outdir
 * </pre>
 * Requires OASIS conformance tests in a directory <TT>conf</TT>
 * under the current directory.
 */
public class OasisTests extends OasisTestDriver {
    /**
     * Package where tests are created.
     */
    private String fTestPackage;

    /**
     * Directory were tests classes are written.
     */
    private String fClassOutputRootDir;

    /**
     * DOM factory to use.
     */
    private String fDomFactoryClassName;

    /**
     * Directory to test output files to.
     */
    private File fOutputDir;

    /**
     * Constructor.
     */
    private OasisTests(String classOutputRootDir,
                       String testPackage,
                       String domFactoryClassName,
                       File outputDir) throws IOException {
        super(XML_CONF);
        fTestPackage = testPackage;
        fClassOutputRootDir = classOutputRootDir;
        fDomFactoryClassName = domFactoryClassName;
        fOutputDir = outputDir;
        fOutputDir.mkdirs();
    }

    /**
     * Create a metadata object with the parameters.  Must specify
     * XML format, as many dest files don't have XML headers.
     */
    private MetaData createMetaData(File xmlFile,
                                    boolean validate,
                                    String className) throws XMLCException {
        MetaDataDocument metaDataDoc = new MetaDataDocument();
        MetaData metaData = metaDataDoc.getMetaData();

        CompileOptions compileOpts = metaData.getCompileOptions();
        compileOpts.setClassOutputRoot(fClassOutputRootDir);

        InputDocument inputDoc = metaData.getInputDocument();
        inputDoc.setUrl(xmlFile.getPath());
        inputDoc.setDocumentFormat(DocumentFormat.XML);
                 
        DocumentClass docClass = metaData.getDocumentClass();
        docClass.setName(className);
        docClass.setDomFactory(fDomFactoryClassName);

        ParserSection parserSection = metaData.getParserSection();
        parserSection.setValidate(validate);

        metaDataDoc.completeModifications();
        return metaData;
    }

    /**
     * Compile a XML file using XMLC.
     */
    private void xmlcCompile(File xmlFile,
                             boolean validate,
                             String className,
                             File errFile) throws IOException, XMLCException {
        PrintWriter errorOut = new PrintWriter(new FileOutputStream(errFile));
        try {
            Compiler compiler = new Compiler(new ErrorReporter(errorOut),
                                             errorOut);
            MetaData metaData = createMetaData(xmlFile, validate, className);
            compiler.compile(metaData);
        } finally {
            errorOut.close();
        }
    }

    /**
     * Make a class name from a test id.
     */
    private String makeClassName(Test test) {
        // Make legal java name.
        String id = test.getId();
        id = id.replace('-', '_');
        return fTestPackage + "." + id;
    }

    /**
     * Compile a test document.  Returns the class name.
     */
    private String compile(Test test) throws IOException, XMLCException {
        String className = makeClassName(test);
        boolean validate = ((test.getType().equals(Test.TYPE_VALID)) ? true : false);
        
        xmlcCompile(test.getTestFile(),
                    validate,
                    className,
                    new File(fOutputDir, test.getId() + ".err"));
        return className;
    }

    /**
     * Run a `valid' test.
     */
    protected void runValidTest(Test test) throws Exception {
        compile(test);
    }

    /**
     * Run the tests.
     */
    private void run() throws IOException, SAXException {
        runTests();
        System.out.println("Succesful tests: " + getSuccessCount());
        System.out.println("Failed tests: " + getFailedCount());
        System.out.println("Disabled test cases: " + getDisabledTestCaseCount());
        System.out.println("Disabled tests: " + getDisabledTestCount());
        System.out.println("Skipped tests: " + getSkippedCount());
            
        if (getFailedCount() > 0) {
            System.err.println("*** " + getFailedCount() + " tests failed");
            System.exit(1);
        }
    }

    /*
     * Test entry point.
     */
    public static void main(String[] args) {
        if (args.length != 4) {
            System.err.println("wrong # args: OasisTests classdir testpkg domfactory outdir");
            System.exit(1);
        }

        try {
            OasisTests tests = new OasisTests(args[0], args[1],
                                              args[2], new File(args[3]));
            tests.run();
        } catch (Throwable err) {
            System.err.println(err.toString());
            err.printStackTrace();
            System.exit(2);
        }
    }
}

/*
 * Local Variables:
 * mode: java
 * End:
 */

