/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: BasicTests.java,v 1.1.1.1 2003/03/10 16:37:12 taweili Exp $
 */

package org.enhydra.xml.xmlc.ssi;

import org.enhydra.xml.driver.TestCaseBase;
import org.enhydra.xml.driver.TestDiff;
import org.enhydra.xml.driver.TestException;
import org.enhydra.xml.driver.TestFileOps;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import java.io.IOException;
import java.io.Writer;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.io.File;
import java.lang.reflect.Method;
import junit.framework.Test;

/**
 * Test of SSI parsing.
 */
public class BasicTests extends TestCaseBase {
    /** Factory method to create suite of these tests */
    public static Test suite() {
        return createSuite(BasicTests.class, null);
    }
    
    /** Required Constructor */
    public BasicTests(Method method) {
        super(method);
    }
    
    /**
     * Test 1: readsize 13
     */
    public void test1() {
        SSIRead test = new SSIRead(this, 13);
	test.runOkTest("basic-top.html");
    }

    /**
     * Test 2: readsize 36
     */
    public void test2() {
        SSIRead test = new SSIRead(this, 36);
	test.runOkTest("basic-top.html");
    }

    /**
     * Test 3: readsize 1300
     */
    public void test3() {
        SSIRead test = new SSIRead(this, 1300);
	test.runOkTest("basic-top.html");
    }

    /**
     * Test 4: readsize 8
     */
    public void test4() {
        SSIRead test = new SSIRead(this, 8);
	test.runOkTest("basic-top.html");
    }

    /**
     * Test 5: single character read
     */
    public void test5() {
        SSIRead test = new SSIRead(this, 1);
	test.runOkTest("basic-top.html");
    }

    /**
     * Test 6: nested includes
     */
    public void test6() {
        SSIRead test = new SSIRead(this);
	test.runOkTest("corner-1.html");
    }

    /**
     * Test 7: Corner case on file content
     */
    public void test7() {
        SSIRead test = new SSIRead(this);
	test.runOkTest("corner-2.html");
    }

    /**
     * Test 8: Corner case on file content
     */
    public void test8() {
        SSIRead test = new SSIRead(this);
	test.runOkTest("corner-3.html");
    }

    /**
     * Test 9: Corner case on file content
     */
    public void test9() {
        SSIRead test = new SSIRead(this);
	test.runOkTest("corner-4.html");
    }

    /**
     * Test 10: Corner case on file content
     */
    public void test10() {
        SSIRead test = new SSIRead(this);
	test.runOkTest("corner-5.html");
    }

    /**
     * Test 11: Corner case on file content
     */
    public void test11() {
        SSIRead test = new SSIRead(this);
	test.runOkTest("large1-top.html");
    }

    /**
     * Test 12: Corner case on SSI directive
     */
    public void test12() {
        SSIRead test = new SSIRead(this);
	test.runOkTest("legal-ssi-1.html");
    }

    /**
     * Test 13: Corner case on SSI directive
     */
    public void test13() {
        SSIRead test = new SSIRead(this);
	test.runOkTest("legal-ssi-2.html");
    }

    /**
     * Test 14: Corner case on SSI directive
     */
    public void test14() {
        SSIRead test = new SSIRead(this);
	test.runOkTest("legal-ssi-3.html");
    }

    /**
     * Test 15: Corner case on SSI directive
     */
    public void test15() {
        SSIRead test = new SSIRead(this);
	test.runOkTest("legal-ssi-4.html");
    }

    /**
     * Test 16: Relative include
     */
    public void test16() {
        SSIRead test = new SSIRead(this);
	test.runOkTest("one/two/rel-1.html");
    }

    /**
     * Test 17: Relative include
     */
    public void test17() {
        SSIRead test = new SSIRead(this);
	test.runOkTest("rel-2.html");
    }

    /**
     * Test 18: Multiple includes in the same directive
     */
    public void test18() {
        SSIRead test = new SSIRead(this);
	test.runOkTest("multiple-1.html");
    }

    /**
     * Test 19: Multiple includes in the same directive
     */
    public void test19() {
        SSIRead test = new SSIRead(this);
	test.runOkTest("multiple-2.html");
    }

    /**
     * Test 20: Multiple includes in the same directive
     */
    public void test20() {
        SSIRead test = new SSIRead(this);
	test.runOkTest("multiple-3.html");
    }

    /**
     * Test 21: Another failure due to SSI alignment
     */
    public void test21() {
        SSIRead test = new SSIRead(this);
	test.runOkTest("seetheerror.html");
    }
}
