/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: WMLTests.java,v 1.1.1.1 2003/03/10 16:37:13 taweili Exp $
 */

package org.enhydra.xml.xmlc.xml;
import org.enhydra.xml.driver.TestDiff;
import org.enhydra.xml.xmlc.driver.XmlcTestParams;
import org.enhydra.xml.xmlc.driver.ExecXmlc;
import org.enhydra.xml.io.OutputOptions;
import java.lang.reflect.Method;
import junit.framework.Test;

/**
 * Test related to WML.  This also tests several output options.
 */
public class WMLTests extends XmlTestCaseBase {

    /** Factory method to create suite of these tests */
    public static Test suite() {
        return createSuite(WMLTests.class, null);
    }
    
    /** Constructor */
    public WMLTests(Method method) {
        super(method);
    }

    /** Create an test forf a WML document. */
    private XmlBasicTest createWMLTest(String relInputDoc) {
        return new XmlBasicTest(this, getInputFile(relInputDoc),
                                getInputFile("wml/wml_1.1.test.xml"));
    }

    /** 
     * Test 1: Encoded as ASCII, but omitting the encoding header.
     * Encoding as ascii encodes non-ascii characters as &#nnn;
     */
    public void test1() {
        XmlBasicTest test = createWMLTest("wml/index.wml");
        OutputOptions options = test.getOutputOptions();
        options.setEncoding("ASCII");
        options.setOmitEncoding(true);
        test.basicTest();
            
        String expectEntities = "&#27665;&#30526;";
        TestDiff differ = getDiffer();
	differ.fgrepMustFind(expectEntities, test.getGenerateFile());
	differ.fgrepMustNotFind("encoding", test.getGenerateFile());
    }

    /** 
     * Test 2: Specified encoding not being lost with toDocument()
     */
    public void test2() {
        // OutputDocument uses toDocument() by default if no options
        // are specified
        XmlBasicTest test = createWMLTest("wml/encoding.wml");
        test.setUseToDocument();
        test.basicTest();
    }

    /** 
     * Test 3: &amp; in attribute values.
     */
    public void test3() {
        XmlBasicTest test = createWMLTest("wml/Login.wml");
        test.basicTest();
    }

    /** 
     * Test 4: Test of empty tag content: <br></br>, as opposed to <br/>
     */
    public void test4() {
        XmlBasicTest test = createWMLTest("wml/Login2.wml");
        test.basicTest();
    }
}
